"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("../action");
class HtmlElementAction extends action_1.Action {
    constructor(testResults, image, locators, boundingBox, id) {
        super(testResults, image, id);
        this.locators = locators;
        this.boundingBox = boundingBox;
    }
    addTestResult(testResult) {
        this.testResults.push(testResult);
    }
    addLocator(locator) {
        this.locators.push(locator);
    }
    recommendedLocator() {
        const locatorCounts = [];
        for (const locator of this.locators) {
            const count = locator.replayableTestResultCount();
            locatorCounts.push({ locator, count });
        }
        // sort desc
        locatorCounts.sort((a, b) => b.count - a.count);
        for (const locatorCount of locatorCounts) {
            if (locatorCount.locator.replayable) {
                return locatorCount.locator;
            }
        }
        return null;
    }
    test(browser, driver) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const locator of this.locators) {
                yield locator.test(driver);
            }
            const recommendedLocator = this.recommendedLocator();
            if (recommendedLocator) {
                const element = yield recommendedLocator.findElement(driver);
                yield this.testElement(driver, element);
            }
            else {
                throw new Error(`No recommended Locator found for Action ${this.constructor.name}`);
            }
        });
    }
}
exports.HtmlElementAction = HtmlElementAction;
